Package: AnVILPublish
Title: Publish Packages and Other Resources to AnVIL Workspaces
Version: 1.21.0
Authors@R: c(
    person(
        "Marcel", "Ramos", role = c("aut", "cre"),
        email = "marcel.ramos@sph.cuny.edu",
        comment = c(ORCID = "0000-0002-3242-0582")
    ),
    person(
        "Martin", "Morgan", role = c("aut"),
        comment = c(ORCID = "0000-0002-5874-8148")
    ),
    person("Kayla", "Interdonato", role = "aut"),
    person(
        "Vincent", "Carey", role = "ctb",
        comment = c(ORCID = "0000-0003-4046-0063")
    ))
Description: 
    Use this package to create or update AnVIL workspaces from
    resources such as R / Bioconductor packages. The metadata about
    the package (e.g., select information from the package DESCRIPTION
    file and from vignette YAML headings) are used to populate the
    'DASHBOARD'. Vignettes are translated to python notebooks ready
    for evaluation in AnVIL.
License: Artistic-2.0
Imports: AnVIL, AnVILGCP, BiocBaseUtils, BiocManager, httr, jsonlite,
        rmarkdown, yaml, readr, whisker, tools, utils, stats
Suggests: knitr, BiocStyle, GCPtools, testthat (>= 3.0.0)
biocViews: Infrastructure, Software
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Config/testthat/edition: 3
Date: 2025-08-12
git_url: https://git.bioconductor.org/packages/AnVILPublish
git_branch: devel
git_last_commit: aafcf59
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 20:02:56 UTC; biocbuild
Author: Marcel Ramos [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3242-0582>),
  Martin Morgan [aut] (ORCID: <https://orcid.org/0000-0002-5874-8148>),
  Kayla Interdonato [aut],
  Vincent Carey [ctb] (ORCID: <https://orcid.org/0000-0003-4046-0063>)
Maintainer: Marcel Ramos <marcel.ramos@sph.cuny.edu>
